# Generated by Django 4.1.10 on 2023-10-10 15:47
from django.db import migrations
from django.db import models


class Migration(migrations.Migration):
    dependencies = [
        ("ipfabric_netbox", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="ipfabricsnapshot",
            name="status",
            field=models.CharField(default="unloaded", max_length=50),
        ),
    ]
