# Generated by Django 4.1.10 on 2024-01-09 16:01
from django.db import migrations
from django.db import models


class Migration(migrations.Migration):
    dependencies = [
        ("ipfabric_netbox", "0003_ipfabricsource_type_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="ipfabricsync",
            name="auto_merge",
            field=models.BooleanField(default=False),
        ),
    ]
